@@ -139,7 +139,11 @@ class PrintInspections(AbstractCommandLineInterface):
         all_inspections: List[Inspection] = []
         for task_id in task_id_list:
             try:
-                task, _ = self.service.api.get_task(project_id, task_id)
+                task = self.service.wrapper.get_task_or_none(project_id, task_id)
+                if task is None:
+                    logger.warning(f"タスク'{task_id}'は存在しないので、スキップします。")
+                    continue
+
                 input_data_id_list = task["input_data_id_list"]
                 logger.info(f"タスク '{task_id}' に紐づく検査コメントを取得します。input_dataの個数 = {len(input_data_id_list)}")
                 for input_data_index, input_data_id in enumerate(input_data_id_list):
@@ -153,9 +157,8 @@ class PrintInspections(AbstractCommandLineInterface):
 
                     all_inspections.extend(inspections)
 
-            except requests.HTTPError as e:
-                logger.warning(e)
-                logger.warning(f"タスク task_id = {task_id} の検査コメントを取得できなかった。")
+            except requests.HTTPError:
+                logger.warning(f"タスク task_id = {task_id} の検査コメントを取得できなかった。", exc_info=True)
 
         return all_inspections
 
@@ -177,7 +180,7 @@ def parse_args(parser: argparse.ArgumentParser):
     argument_parser.add_project_id()
     argument_parser.add_task_id(
         required=True,
-        help_message="対象のタスクのtask_idを指定します。　" "`file://`を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。",
+        help_message="対象のタスクのtask_idを指定します。　" " ``file://`` を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。",
     )
 
     reply_comment_group = parser.add_mutually_exclusive_group()
@@ -206,7 +209,7 @@ def main(args):
     PrintInspections(service, facade, args).main()
 
 
-def add_parser(subparsers: argparse._SubParsersAction):
+def add_parser(subparsers: Optional[argparse._SubParsersAction] = None):
     subcommand_name = "list"
 
     subcommand_help = "検査コメント一覧を出力します。"
@@ -215,3 +218,4 @@ def add_parser(subparsers: argparse._SubParsersAction):
 
     parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description)
     parse_args(parser)
+    return parser
