@@ -3,131 +3,160 @@ import logging
 from pathlib import Path
 from typing import Dict, List, Optional, Set
 
-import pandas
+from annofabapi.models import TaskPhase
 
 import annofabcli
 from annofabcli.common.cli import get_list_from_args
-from annofabcli.common.utils import print_csv, read_multiheader_csv
 from annofabcli.filesystem.mask_user_info import (
     create_masked_user_info_df,
     create_replacement_dict_by_user_id,
     replace_by_columns,
 )
-from annofabcli.stat_visualization.write_linegraph_per_user import write_linegraph_per_user
-from annofabcli.stat_visualization.write_performance_scatter_per_user import write_performance_scatter_per_user
-from annofabcli.statistics.csv import FILENAME_PERFORMANCE_PER_USER, FILENAME_TASK_LIST
+from annofabcli.statistics.visualization.dataframe.cumulative_productivity import (
+    AcceptorCumulativeProductivity,
+    AnnotatorCumulativeProductivity,
+    InspectorCumulativeProductivity,
+)
+from annofabcli.statistics.visualization.dataframe.productivity_per_date import (
+    AcceptorProductivityPerDate,
+    AnnotatorProductivityPerDate,
+    InspectorProductivityPerDate,
+)
+from annofabcli.statistics.visualization.dataframe.task import Task
+from annofabcli.statistics.visualization.dataframe.user_performance import UserPerformance
 from annofabcli.statistics.visualization.dataframe.worktime_per_date import WorktimePerDate
+from annofabcli.statistics.visualization.project_dir import ProjectDir
 
 logger = logging.getLogger(__name__)
 
 
-def _replace_df_task(df, replacement_dict_by_user_id: Dict[str, str]):
-    replace_by_columns(df, replacement_dict_by_user_id, main_column="user_id", sub_columns=["username"])
+def _replace_df_task(task: Task, replacement_dict_by_user_id: Dict[str, str]) -> Task:
+    df_output = task.df.copy()
+    replace_by_columns(df_output, replacement_dict_by_user_id, main_column="user_id", sub_columns=["username"])
 
     replace_by_columns(
-        df,
+        df_output,
         replacement_dict_by_user_id,
         main_column="first_annotation_user_id",
         sub_columns=["first_annotation_username"],
     )
     replace_by_columns(
-        df,
+        df_output,
         replacement_dict_by_user_id,
         main_column="first_inspection_user_id",
         sub_columns=["first_inspection_username"],
     )
     replace_by_columns(
-        df,
+        df_output,
         replacement_dict_by_user_id,
         main_column="first_acceptance_user_id",
         sub_columns=["first_acceptance_username"],
     )
+    return Task(df_output)
 
 
-# class MaskingVisualizationDir:
-#     def __init__(self,     not_masked_biography_set: Optional[Set[str]] = None,
-#         not_masked_user_id_set: Optional[Set[str]] = None,
-#         minimal_output: bool = False,
-#         exclude_masked_user_for_linegraph: bool = False,
-#     ) -> None:
-#         self.not_masked_biography_set
+def write_line_graph(
+    task: Task, output_project_dir: ProjectDir, user_id_list: Optional[List[str]] = None, minimal_output: bool = False
+):
+    df = task.df.copy()
+
+    output_project_dir.write_cumulative_line_graph(
+        AnnotatorCumulativeProductivity(df),
+        phase=TaskPhase.ANNOTATION,
+        user_id_list=user_id_list,
+        minimal_output=minimal_output,
+    )
+    output_project_dir.write_cumulative_line_graph(
+        InspectorCumulativeProductivity(df),
+        phase=TaskPhase.INSPECTION,
+        user_id_list=user_id_list,
+        minimal_output=minimal_output,
+    )
+    output_project_dir.write_cumulative_line_graph(
+        AcceptorCumulativeProductivity(df),
+        phase=TaskPhase.ACCEPTANCE,
+        user_id_list=user_id_list,
+        minimal_output=minimal_output,
+    )
+
+    annotator_per_date_obj = AnnotatorProductivityPerDate.from_df_task(task.df)
+    inspector_per_date_obj = InspectorProductivityPerDate.from_df_task(task.df)
+    acceptor_per_date_obj = AcceptorProductivityPerDate.from_df_task(task.df)
+
+    output_project_dir.write_performance_per_started_date_csv(annotator_per_date_obj, phase=TaskPhase.ANNOTATION)
+    output_project_dir.write_performance_per_started_date_csv(inspector_per_date_obj, phase=TaskPhase.INSPECTION)
+    output_project_dir.write_performance_per_started_date_csv(acceptor_per_date_obj, phase=TaskPhase.ACCEPTANCE)
+
+    if not minimal_output:
+        output_project_dir.write_performance_line_graph_per_date(
+            annotator_per_date_obj, phase=TaskPhase.ANNOTATION, user_id_list=user_id_list
+        )
+        output_project_dir.write_performance_line_graph_per_date(
+            inspector_per_date_obj, phase=TaskPhase.INSPECTION, user_id_list=user_id_list
+        )
+        output_project_dir.write_performance_line_graph_per_date(
+            acceptor_per_date_obj, phase=TaskPhase.ACCEPTANCE, user_id_list=user_id_list
+        )
 
 
 def mask_visualization_dir(
-    project_dir: Path,
-    output_dir: Path,
+    project_dir: ProjectDir,
+    output_project_dir: ProjectDir,
     *,
     not_masked_biography_set: Optional[Set[str]] = None,
     not_masked_user_id_set: Optional[Set[str]] = None,
     minimal_output: bool = False,
     exclude_masked_user_for_linegraph: bool = False,
 ):
-    if not (project_dir / FILENAME_PERFORMANCE_PER_USER).exists():
-        logger.warning(
-            f"'{str(project_dir / FILENAME_PERFORMANCE_PER_USER)}'が存在しないので、'{str(project_dir)}'のマスク処理をスキップします。"
-        )
-        return
-
-    df_member_performance = read_multiheader_csv(str(project_dir / FILENAME_PERFORMANCE_PER_USER), header_row_count=2)
+    # TODO: validation
+    user_performance = project_dir.read_user_performance()
 
+    # マスクするユーザの情報を取得する
     replacement_dict_by_user_id = create_replacement_dict_by_user_id(
-        df_member_performance,
+        user_performance.df,
         not_masked_biography_set=not_masked_biography_set,
         not_masked_user_id_set=not_masked_user_id_set,
     )
-    not_masked_user_id_set = set(df_member_performance[("user_id", "")]) - set(replacement_dict_by_user_id.keys())
+    not_masked_user_id_set = set(user_performance.df[("user_id", "")]) - set(replacement_dict_by_user_id.keys())
 
     # CSVのユーザ情報をマスクする
     masked_df_member_performance = create_masked_user_info_df(
-        df_member_performance,
+        user_performance.df,
         not_masked_biography_set=not_masked_biography_set,
         not_masked_user_id_set=not_masked_user_id_set,
     )
-    print_csv(masked_df_member_performance, output=str(output_dir / FILENAME_PERFORMANCE_PER_USER))
+    masked_user_performance = UserPerformance(masked_df_member_performance)
+    output_project_dir.write_user_performance(masked_user_performance)
 
     # メンバのパフォーマンスを散布図で出力する
-    write_performance_scatter_per_user(output_dir / FILENAME_PERFORMANCE_PER_USER, output_dir=output_dir / "scatter")
+    output_project_dir.write_user_performance_scatter_plot(masked_user_performance)
 
     user_id_list: Optional[List[str]] = None
     if exclude_masked_user_for_linegraph:
         user_id_list = list(not_masked_user_id_set)
 
-    task_csv_file = project_dir / FILENAME_TASK_LIST
-    if task_csv_file.exists():
-        df_task = pandas.read_csv(str(project_dir / FILENAME_TASK_LIST))
-        _replace_df_task(df_task, replacement_dict_by_user_id=replacement_dict_by_user_id)
-        print_csv(df_task, output=str(output_dir / FILENAME_TASK_LIST))
+    # TODO: validation
+    task = project_dir.read_task_list()
+    masked_task = _replace_df_task(task, replacement_dict_by_user_id=replacement_dict_by_user_id)
+    output_project_dir.write_task_list(masked_task)
+
+    write_line_graph(masked_task, output_project_dir, user_id_list=user_id_list, minimal_output=minimal_output)
+
+    try:
+        worktime_per_date_user = project_dir.read_worktime_per_date_user()
+    except Exception:
+        logger.warning(f"'{project_dir}'のユーザごと日ごとの作業時間の読み込みに失敗しました。")
     else:
-        logger.warning(f"'{task_csv_file}' が存在しないため、" f"'{output_dir / FILENAME_TASK_LIST}' は出力しません。 ")
-
-    if (output_dir / FILENAME_TASK_LIST).exists():
-        # メンバごとにパフォーマンスを折れ線グラフで出力する
-        write_linegraph_per_user(
-            output_dir / FILENAME_TASK_LIST,
-            output_dir=output_dir / "line-graph",
-            minimal_output=minimal_output,
-            user_id_list=user_id_list,
-        )
-
-    user_date_csv_file = project_dir / "ユーザ_日付list-作業時間.csv"
-    if user_date_csv_file.exists():
-        df_worktime = pandas.read_csv(str(user_date_csv_file))
         df_masked_worktime = create_masked_user_info_df(
-            df_worktime,
+            worktime_per_date_user.df,
             not_masked_biography_set=not_masked_biography_set,
             not_masked_user_id_set=not_masked_user_id_set,
         )
-        worktime_per_date_obj = WorktimePerDate(df_masked_worktime)
-        worktime_per_date_obj.plot_cumulatively(output_dir / "line-graph/累積折れ線-横軸_日-縦軸_作業時間.html", user_id_list)
-        worktime_per_date_obj.to_csv(output_dir / "ユーザ_日付list-作業時間.csv")
-    else:
-        logger.warning(
-            f"{user_date_csv_file}が存在しないため、"
-            f"'{output_dir / 'line-graph/累積折れ線-横軸_日-縦軸_作業時間.html'}', "
-            f"'{output_dir / 'ユーザ_日付list-作業時間.csv'}' は出力しません。"
-        )
+        masked_worktime_per_date_user = WorktimePerDate(df_masked_worktime)
+        output_project_dir.write_worktime_per_date_user(masked_worktime_per_date_user)
+        output_project_dir.write_worktime_line_graph(masked_worktime_per_date_user, user_id_list=user_id_list)
 
-    logger.debug(f"'{project_dir}'のマスクした結果を'{output_dir}'に出力しました。")
+    logger.debug(f"'{project_dir}'のマスクした結果を'{output_project_dir}'に出力しました。")
 
 
 def mask_visualization_root_dir(
@@ -139,13 +168,14 @@ def mask_visualization_root_dir(
     exclude_masked_user_for_linegraph: bool = False,
 ):
 
-    for project_dir in project_root_dir.iterdir():
-        if not project_dir.is_dir():
+    for p_project_dir in project_root_dir.iterdir():
+        if not p_project_dir.is_dir():
             continue
 
-        project_output_dir = output_dir / project_dir.name
+        project_output_dir = ProjectDir(output_dir / p_project_dir.name)
 
         try:
+            project_dir = ProjectDir(p_project_dir)
             mask_visualization_dir(
                 project_dir,
                 project_output_dir,
@@ -183,8 +213,7 @@ def parse_args(parser: argparse.ArgumentParser):
         "--dir",
         type=Path,
         required=True,
-        help=f"マスクしたいプロジェクトディレクトリが存在するディレクトリを指定してください。プロジェクトディレクトリは  ``annofabcli statistics visualize`` コマンドの出力結果です。\n"
-        f"プロジェクトディレクトリ配下の'{FILENAME_PERFORMANCE_PER_USER}'を読み込み、ユーザ情報をマスクします。",
+        help=f"マスクしたいプロジェクトディレクトリが存在するディレクトリを指定してください。プロジェクトディレクトリは  ``annofabcli statistics visualize`` コマンドの出力結果です。",  # noqa: E501
     )
 
     parser.add_argument(
