@@ -3,14 +3,9 @@ from typing import Optional
 
 import annofabcli.stat_visualization.mask_visualization_dir
 import annofabcli.stat_visualization.merge_visualization_dir
+import annofabcli.stat_visualization.write_graph
 import annofabcli.stat_visualization.write_performance_rating_csv
-from annofabcli.stat_visualization import (
-    summarise_whole_performance_csv,
-    write_linegraph_per_user,
-    write_performance_scatter_per_user,
-    write_task_histogram,
-    write_whole_linegraph,
-)
+from annofabcli.stat_visualization import summarise_whole_performance_csv
 
 
 def parse_args(parser: argparse.ArgumentParser):
@@ -21,11 +16,8 @@ def parse_args(parser: argparse.ArgumentParser):
     annofabcli.stat_visualization.mask_visualization_dir.add_parser(subparsers)
     annofabcli.stat_visualization.merge_visualization_dir.add_parser(subparsers)
     summarise_whole_performance_csv.add_parser(subparsers)
-    write_linegraph_per_user.add_parser(subparsers)
-    write_performance_scatter_per_user.add_parser(subparsers)
+    annofabcli.stat_visualization.write_graph.add_parser(subparsers)
     annofabcli.stat_visualization.write_performance_rating_csv.add_parser(subparsers)
-    write_task_histogram.add_parser(subparsers)
-    write_whole_linegraph.add_parser(subparsers)
 
 
 def add_parser(subparsers: Optional[argparse._SubParsersAction] = None):
