@@ -28,10 +28,18 @@ def write_bokeh_graph(bokeh_obj, output_file: Path):
     bokeh.plotting.reset_output()
     bokeh.plotting.output_file(output_file, title=output_file.stem)
     bokeh.plotting.save(bokeh_obj)
+    logger.debug(f"'{output_file}'を出力しました。")
 
 
 def get_weekly_moving_average(series: pandas.Series) -> pandas.Series:
-    """1週間移動平均をプロットするために、1週間移動平均用のpandas.Seriesを取得する。"""
+    """1週間移動平均用のpandas.Seriesを取得する。"""
+    MOVING_WINDOW_DAYS = 7
+    MIN_WINDOW_DAYS = 2
+    return series.rolling(MOVING_WINDOW_DAYS, min_periods=MIN_WINDOW_DAYS).mean()
+
+
+def get_weekly_sum(series: pandas.Series) -> pandas.Series:
+    """1週間の合計値が格納されたpandas.Seriesを取得する。"""
     MOVING_WINDOW_DAYS = 7
     MIN_WINDOW_DAYS = 2
     return series.rolling(MOVING_WINDOW_DAYS, min_periods=MIN_WINDOW_DAYS).sum()
