@@ -1,17 +1,16 @@
 import argparse
+import functools
 import logging.handlers
 import re
 import sys
 import tempfile
-from dataclasses import dataclass
 from multiprocessing import Pool
 from pathlib import Path
 from typing import Any, Callable, Collection, List, Optional
 
 import annofabapi
 import pandas
-from annofabapi.models import ProjectMemberRole
-from dataclasses_json import DataClassJsonMixin
+from annofabapi.models import ProjectMemberRole, TaskPhase
 
 import annofabcli
 from annofabcli.common.cli import (
@@ -21,12 +20,6 @@ from annofabcli.common.cli import (
 )
 from annofabcli.common.facade import AnnofabApiFacade, TaskQuery
 from annofabcli.stat_visualization.merge_visualization_dir import merge_visualization_dir
-from annofabcli.statistics.csv import (
-    FILENAME_PERFORMANCE_PER_DATE,
-    FILENAME_PERFORMANCE_PER_USER,
-    FILENAME_TASK_LIST,
-    FILENAME_WHOLE_PERFORMANCE,
-)
 from annofabcli.statistics.database import Database, Query
 from annofabcli.statistics.table import Table
 from annofabcli.statistics.visualization.dataframe.cumulative_productivity import (
@@ -39,55 +32,23 @@ from annofabcli.statistics.visualization.dataframe.productivity_per_date import
     AnnotatorProductivityPerDate,
     InspectorProductivityPerDate,
 )
-from annofabcli.statistics.visualization.dataframe.task import Task
-from annofabcli.statistics.visualization.dataframe.user_performance import (
+from annofabcli.statistics.visualization.dataframe.project_performance import (
     ProjectPerformance,
-    UserPerformance,
-    WholePerformance,
+    ProjectWorktimePerMonth,
 )
+from annofabcli.statistics.visualization.dataframe.task import Task
+from annofabcli.statistics.visualization.dataframe.user_performance import UserPerformance, WholePerformance
 from annofabcli.statistics.visualization.dataframe.whole_productivity_per_date import (
     WholeProductivityPerCompletedDate,
     WholeProductivityPerFirstAnnotationStartedDate,
 )
 from annofabcli.statistics.visualization.dataframe.worktime_per_date import WorktimePerDate
+from annofabcli.statistics.visualization.model import WorktimeColumn
+from annofabcli.statistics.visualization.project_dir import ProjectDir, ProjectInfo
 
 logger = logging.getLogger(__name__)
 
 
-@dataclass
-class CommnadLineArgs(DataClassJsonMixin):
-    task_query: Optional[TaskQuery]
-    user_id_list: Optional[List[str]]
-    start_date: Optional[str]
-    end_date: Optional[str]
-
-
-@dataclass
-class ProjectSummary(DataClassJsonMixin):
-    project_id: str
-    project_title: str
-    measurement_datetime: str
-    """計測日時。（2004-04-01T12:00+09:00形式）"""
-    args: CommnadLineArgs
-    """コマンドライン引数"""
-
-
-def write_project_performance_csv(project_dirs: Collection[Path], output_path: Path):
-    csv_file_list = [e / FILENAME_WHOLE_PERFORMANCE for e in project_dirs]
-
-    obj_list = []
-    project_title_list = []
-    for csv_file in csv_file_list:
-        if csv_file.exists():
-            obj_list.append(WholePerformance.from_csv(csv_file))
-            project_title_list.append(csv_file.parent.name)
-        else:
-            logger.warning(f"{csv_file} は存在しないのでスキップします。")
-
-    main_obj = ProjectPerformance.from_whole_performance_objs(obj_list, project_title_list)
-    main_obj.to_csv(output_path)
-
-
 def get_project_output_dir(project_title: str) -> str:
     return re.sub(r'[\\/:*?"<>|]+', "__", project_title)
 
@@ -103,6 +64,8 @@ class WriteCsvGraph:
         table_obj: Table,
         output_dir: Path,
         df_labor: Optional[pandas.DataFrame],
+        start_date: Optional[str] = None,
+        end_date: Optional[str] = None,
         minimal_output: bool = False,
         output_only_text: bool = False,
     ):
@@ -111,9 +74,13 @@ class WriteCsvGraph:
         self.output_dir = output_dir
         self.table_obj = table_obj
         self.df_labor = df_labor
+        self.start_date = start_date
+        self.end_date = end_date
         self.minimal_output = minimal_output
         self.output_only_text = output_only_text
 
+        self.project_dir = ProjectDir(output_dir)
+
     def _catch_exception(self, function: Callable[..., Any]) -> Callable[..., Any]:
         """
         Exceptionをキャッチしてログにstacktraceを出力する。
@@ -145,11 +112,11 @@ class WriteCsvGraph:
 
         """
         obj = Task(self._get_task_df())
-        obj.to_csv(self.output_dir / FILENAME_TASK_LIST)
+
+        self.project_dir.write_task_list(obj)
 
         if not self.output_only_text:
-            obj.plot_histogram_of_worktime(self.output_dir / "histogram/ヒストグラム-作業時間.html")
-            obj.plot_histogram_of_others(self.output_dir / "histogram/ヒストグラム.html")
+            self.project_dir.write_task_histogram(obj)
 
     def write_user_performance(self) -> None:
         """
@@ -161,39 +128,21 @@ class WriteCsvGraph:
             df_task_history, self._get_task_df()
         )
         if len(annotation_count_ratio_df) == 0:
-            obj = UserPerformance(pandas.DataFrame())
+            user_performance = UserPerformance(pandas.DataFrame())
         else:
-            obj = UserPerformance.from_df(
+            user_performance = UserPerformance.from_df(
                 df_task_history=df_task_history,
                 df_labor=self.df_labor,
                 df_worktime_ratio=annotation_count_ratio_df,
             )
 
-        obj.to_csv(self.output_dir / FILENAME_PERFORMANCE_PER_USER)
-        WholePerformance(obj.get_summary()).to_csv(self.output_dir / FILENAME_WHOLE_PERFORMANCE)
+        self.project_dir.write_user_performance(user_performance)
+
+        whole_performance = WholePerformance.from_user_performance(user_performance)
+        self.project_dir.write_whole_performance(whole_performance)
 
         if not self.output_only_text:
-            obj.plot_quality(self.output_dir / "scatter/散布図-教師付者の品質と作業量の関係.html")
-            obj.plot_productivity_from_monitored_worktime(
-                self.output_dir / "scatter/散布図-アノテーションあたり作業時間と累計作業時間の関係-計測時間.html"
-            )
-            obj.plot_quality_and_productivity_from_monitored_worktime(
-                self.output_dir / "scatter/散布図-アノテーションあたり作業時間と品質の関係-計測時間-教師付者用.html"
-            )
-
-            if obj.actual_worktime_exists():
-                obj.plot_productivity_from_actual_worktime(
-                    self.output_dir / "scatter/散布図-アノテーションあたり作業時間と累計作業時間の関係-実績時間.html"
-                )
-                obj.plot_quality_and_productivity_from_actual_worktime(
-                    self.output_dir / "scatter/散布図-アノテーションあたり作業時間と品質の関係-実績時間-教師付者用.html"
-                )
-            else:
-                logger.warning(
-                    f"実績作業時間の合計値が0なので、実績作業時間関係の以下のグラフは出力しません。\n"
-                    " * '散布図-アノテーションあたり作業時間と累計作業時間の関係-実績時間.html'\n"
-                    " * '散布図-アノテーションあたり作業時間と品質の関係-実績時間-教師付者用'"
-                )
+            self.project_dir.write_user_performance_scatter_plot(user_performance)
 
     def write_cumulative_linegraph_by_user(self, user_id_list: Optional[List[str]] = None) -> None:
         """ユーザごとの累積折れ線グラフをプロットする。"""
@@ -204,86 +153,64 @@ class WriteCsvGraph:
         acceptor_obj = AcceptorCumulativeProductivity(df_task)
 
         if not self.output_only_text:
-            annotator_obj.plot_annotation_metrics(
-                self.output_dir / "line-graph/教師付者用/累積折れ線-横軸_アノテーション数-教師付者用.html", user_id_list
-            )
-            inspector_obj.plot_annotation_metrics(
-                self.output_dir / "line-graph/検査者用/累積折れ線-横軸_アノテーション数-検査者用.html", user_id_list
-            )
-            acceptor_obj.plot_annotation_metrics(
-                self.output_dir / "line-graph/受入者用/累積折れ線-横軸_アノテーション数-受入者用.html", user_id_list
-            )
 
-            if not self.minimal_output:
-                annotator_obj.plot_input_data_metrics(
-                    self.output_dir / "line-graph/教師付者用/累積折れ線-横軸_入力データ数-教師付者用.html", user_id_list
-                )
-                inspector_obj.plot_input_data_metrics(
-                    self.output_dir / "line-graph/検査者用/累積折れ線-横軸_入力データ数-検査者用.html", user_id_list
-                )
-                acceptor_obj.plot_input_data_metrics(
-                    self.output_dir / "line-graph/受入者用/累積折れ線-横軸_入力データ数-受入者用.html", user_id_list
-                )
-
-                annotator_obj.plot_task_metrics(
-                    self.output_dir / "line-graph/教師付者用/累積折れ線-横軸_タスク数-教師付者用.html", user_id_list
-                )
+            self.project_dir.write_cumulative_line_graph(
+                annotator_obj, phase=TaskPhase.ANNOTATION, user_id_list=user_id_list, minimal_output=self.minimal_output
+            )
+            self.project_dir.write_cumulative_line_graph(
+                inspector_obj, phase=TaskPhase.INSPECTION, user_id_list=user_id_list, minimal_output=self.minimal_output
+            )
+            self.project_dir.write_cumulative_line_graph(
+                acceptor_obj, phase=TaskPhase.ACCEPTANCE, user_id_list=user_id_list, minimal_output=self.minimal_output
+            )
 
     def write_worktime_per_date(self, user_id_list: Optional[List[str]] = None) -> None:
         """日ごとの作業時間情報を出力する。"""
-        worktime_per_date_obj = WorktimePerDate.from_webapi(self.service, self.project_id, self.df_labor)
-        worktime_per_date_obj.to_csv(self.output_dir / "ユーザ_日付list-作業時間.csv")
+        worktime_per_date_obj = WorktimePerDate.from_webapi(
+            self.service, self.project_id, self.df_labor, start_date=self.start_date, end_date=self.end_date
+        )
+
+        self.project_dir.write_worktime_per_date_user(worktime_per_date_obj)
 
         df_task = self._get_task_df()
         productivity_per_completed_date_obj = WholeProductivityPerCompletedDate.from_df(
             df_task, worktime_per_date_obj.df
         )
-        productivity_per_completed_date_obj.to_csv(self.output_dir / FILENAME_PERFORMANCE_PER_DATE)
+
+        self.project_dir.write_whole_productivity_per_date(productivity_per_completed_date_obj)
 
         productivity_per_started_date_obj = WholeProductivityPerFirstAnnotationStartedDate.from_df(df_task)
-        productivity_per_started_date_obj.to_csv(self.output_dir / "教師付開始日毎の生産量と生産性.csv")
+        self.project_dir.write_whole_productivity_per_first_annotation_started_date(productivity_per_started_date_obj)
 
         if not self.output_only_text:
-            worktime_per_date_obj.plot_cumulatively(
-                self.output_dir / "line-graph/累積折れ線-横軸_日-縦軸_作業時間.html", user_id_list
+            self.project_dir.write_worktime_line_graph(worktime_per_date_obj, user_id_list=user_id_list)
+            self.project_dir.write_whole_productivity_line_graph_per_date(productivity_per_completed_date_obj)
+            self.project_dir.write_whole_productivity_line_graph_per_annotation_started_date(
+                productivity_per_started_date_obj
             )
-            productivity_per_completed_date_obj.plot(self.output_dir / "line-graph/折れ線-横軸_日-全体.html")
-            productivity_per_completed_date_obj.plot_cumulatively(self.output_dir / "line-graph/累積折れ線-横軸_日-全体.html")
-            productivity_per_started_date_obj.plot(self.output_dir / "line-graph/折れ線-横軸_教師付開始日-全体.html")
 
     def write_user_productivity_per_date(self, user_id_list: Optional[List[str]] = None):
         """ユーザごとの日ごとの生産性情報を出力する。"""
-        # 各ユーザごとの日ごとの情報
         df_task = self._get_task_df()
+
         annotator_per_date_obj = AnnotatorProductivityPerDate.from_df_task(df_task)
-        annotator_per_date_obj.to_csv(self.output_dir / Path("教師付者_教師付開始日list.csv"))
-
         inspector_per_date_obj = InspectorProductivityPerDate.from_df_task(df_task)
-        inspector_per_date_obj.to_csv(self.output_dir / Path("検査者_検査開始日list.csv"))
+        acceptor_per_date_obj = AcceptorProductivityPerDate.from_df_task(df_task)
 
-        acceptor_per_date = AcceptorProductivityPerDate.from_df_task(df_task)
-        acceptor_per_date.to_csv(self.output_dir / Path("受入者_受入開始日list.csv"))
+        # 開始日ごとのCSVを出力
+        self.project_dir.write_performance_per_started_date_csv(annotator_per_date_obj, phase=TaskPhase.ANNOTATION)
+        self.project_dir.write_performance_per_started_date_csv(inspector_per_date_obj, phase=TaskPhase.INSPECTION)
+        self.project_dir.write_performance_per_started_date_csv(acceptor_per_date_obj, phase=TaskPhase.ACCEPTANCE)
 
         if not self.output_only_text:
-            annotator_per_date_obj.plot_annotation_metrics(
-                self.output_dir / Path("line-graph/教師付者用/折れ線-横軸_教師付開始日-縦軸_アノテーション単位の指標-教師付者用.html"), user_id_list
+            self.project_dir.write_performance_line_graph_per_date(
+                annotator_per_date_obj, phase=TaskPhase.ANNOTATION, user_id_list=user_id_list
             )
-            annotator_per_date_obj.plot_input_data_metrics(
-                self.output_dir / Path("line-graph/教師付者用/折れ線-横軸_教師付開始日-縦軸_入力データ単位の指標-教師付者用.html"), user_id_list
+            self.project_dir.write_performance_line_graph_per_date(
+                inspector_per_date_obj, phase=TaskPhase.INSPECTION, user_id_list=user_id_list
             )
-
-            inspector_per_date_obj.plot_annotation_metrics(
-                self.output_dir / Path("line-graph/検査者用/折れ線-横軸_検査開始日-縦軸_アノテーション単位の指標-検査者用.html"), user_id_list
-            )
-            inspector_per_date_obj.plot_input_data_metrics(
-                self.output_dir / Path("line-graph/検査者用/折れ線-横軸_検査開始日-縦軸_入力データ単位の指標-検査者用.html"), user_id_list
-            )
-
-            acceptor_per_date.plot_annotation_metrics(
-                self.output_dir / Path("line-graph/検査者用/折れ線-横軸_検査開始日-縦軸_アノテーション単位の指標-検査者用.html"), user_id_list
-            )
-            acceptor_per_date.plot_input_data_metrics(
-                self.output_dir / Path("line-graph/検査者用/折れ線-横軸_検査開始日-縦軸_入力データ単位の指標-検査者用.html"), user_id_list
+            self.project_dir.write_performance_line_graph_per_date(
+                acceptor_per_date_obj, phase=TaskPhase.ACCEPTANCE, user_id_list=user_id_list
             )
 
 
@@ -322,25 +249,24 @@ class VisualizingStatisticsMain:
         self.df_labor = df_labor
         self.user_ids = user_ids
 
-    def write_project_name_file(self, project_id: str, command_line_args: CommnadLineArgs, output_project_dir: Path):
+    def write_project_info_json(self, project_id: str, project_dir: ProjectDir):
         """
-        ファイル名がプロジェクト名のjsonファイルを生成する。
+        プロジェクト情報をJSONファイルに出力します。
         """
         project_info = self.service.api.get_project(project_id)[0]
         project_title = project_info["title"]
         logger.info(f"project_title = {project_title}")
-        filename = annofabcli.utils.to_filename(project_title)
-        output_project_dir.mkdir(exist_ok=True, parents=True)
 
-        project_summary = ProjectSummary(
+        project_summary = ProjectInfo(
             project_id=project_id,
             project_title=project_title,
+            input_data_type=project_info["input_data_type"],
             measurement_datetime=annofabapi.utils.str_now(),
-            args=command_line_args,
+            query=Query(
+                task_query=self.task_query, task_ids=self.task_ids, start_date=self.start_date, end_date=self.end_date
+            ),
         )
-
-        with open(str(output_project_dir / f"{filename}.json"), "w", encoding="utf-8") as f:
-            f.write(project_summary.to_json(ensure_ascii=False, indent=2))
+        project_dir.write_project_info(project_summary)
 
     def visualize_statistics(
         self,
@@ -360,13 +286,16 @@ class VisualizingStatisticsMain:
             project_id, project_member_roles=[ProjectMemberRole.OWNER, ProjectMemberRole.TRAINING_DATA_USER]
         )
 
+        project_dir = ProjectDir(output_project_dir)
+        self.write_project_info_json(project_id=project_id, project_dir=project_dir)
+
         database = Database(
             self.service,
             project_id,
             self.temp_dir,
             query=Query(
                 task_query=self.task_query,
-                task_id_set=set(self.task_ids) if self.task_ids is not None else None,
+                task_ids=set(self.task_ids) if self.task_ids is not None else None,
                 start_date=self.start_date,
                 end_date=self.end_date,
             ),
@@ -383,23 +312,17 @@ class VisualizingStatisticsMain:
             logger.warning(f"project_id={project_id}: タスク履歴一覧が0件なのでファイルを出力しません。終了します。")
             return
 
-        self.write_project_name_file(
-            project_id=project_id,
-            command_line_args=CommnadLineArgs(
-                task_query=self.task_query,
-                user_id_list=self.user_ids,
-                start_date=self.start_date,
-                end_date=self.end_date,
-            ),
-            output_project_dir=output_project_dir,
-        )
-
         if self.df_labor is not None:
             # project_id列がある場合（複数のproject_id列を指定した場合）はproject_idで絞り込む
             if "project_id" in self.df_labor:
                 df_labor = self.df_labor[self.df_labor["project_id"] == project_id]
             else:
                 df_labor = self.df_labor
+            if self.start_date is not None:
+                df_labor = df_labor[df_labor["date"] >= self.start_date]
+            if self.end_date is not None:
+                df_labor = df_labor[df_labor["date"] <= self.end_date]
+
         else:
             df_labor = None
 
@@ -409,6 +332,8 @@ class VisualizingStatisticsMain:
             table_obj,
             output_project_dir,
             df_labor=df_labor,
+            start_date=self.start_date,
+            end_date=self.end_date,
             minimal_output=self.minimal_output,
             output_only_text=self.output_only_text,
         )
@@ -425,6 +350,23 @@ class VisualizingStatisticsMain:
         if not self.minimal_output:
             write_obj._catch_exception(write_obj.write_user_productivity_per_date)(self.user_ids)
 
+    def visualize_statistics_wrapper(
+        self,
+        project_id: str,
+        root_output_dir: Path,
+    ) -> Optional[Path]:
+        try:
+            project_title = self.facade.get_project_title(project_id)
+            output_project_dir = root_output_dir / get_project_output_dir(project_title)
+            self.visualize_statistics(
+                project_id=project_id,
+                output_project_dir=output_project_dir,
+            )
+            return output_project_dir
+        except Exception:  # pylint: disable=broad-except
+            logger.warning(f"project_id='{project_id}'の可視化処理に失敗しました。", exc_info=True)
+            return None
+
     def visualize_statistics_for_project_list(
         self,
         project_id_list: List[str],
@@ -432,24 +374,10 @@ class VisualizingStatisticsMain:
         *,
         parallelism: Optional[int] = None,
     ) -> List[Path]:
-        def wrap(
-            project_id: str,
-        ) -> Optional[Path]:
-            project_title = self.facade.get_project_title(project_id)
-            output_project_dir = root_output_dir / get_project_output_dir(project_title)
-
-            try:
-                self.visualize_statistics(
-                    project_id=project_id,
-                    output_project_dir=output_project_dir,
-                )
-                return output_project_dir
-            except Exception:  # pylint: disable=broad-except
-                logger.warning(f"project_id='{project_id}'の可視化処理に失敗しました。")
-                return None
 
         output_project_dir_list: List[Path] = []
 
+        wrap = functools.partial(self.visualize_statistics_wrapper, root_output_dir=root_output_dir)
         if parallelism is not None:
             with Pool(parallelism) as pool:
                 result_list = pool.map(wrap, project_id_list)
@@ -533,14 +461,27 @@ class VisualizeStatistics(AbstractCommandLineInterface):
 
             if args.merge:
                 merge_visualization_dir(
-                    project_dir_list=output_project_dir_list,
-                    output_dir=root_output_dir / "merge",
+                    project_dir_list=[ProjectDir(e) for e in output_project_dir_list],
+                    output_project_dir=ProjectDir(root_output_dir / "merge"),
                     user_id_list=user_id_list,
                     minimal_output=args.minimal,
                 )
 
             if len(output_project_dir_list) > 0:
-                write_project_performance_csv(output_project_dir_list, root_output_dir / "プロジェクトごとの生産性と品質.csv")
+                project_dir_list = [ProjectDir(e) for e in output_project_dir_list]
+                project_performance = ProjectPerformance.from_project_dirs(project_dir_list)
+                project_performance.to_csv(root_output_dir / "プロジェクトごとの生産性と品質.csv")
+
+                project_actual_worktime = ProjectWorktimePerMonth.from_project_dirs(
+                    project_dir_list, WorktimeColumn.ACTUAL_WORKTIME_HOUR
+                )
+                project_actual_worktime.to_csv(root_output_dir / "プロジェクごとの毎月の実績作業時間.csv")
+
+                project_monitored_worktime = ProjectWorktimePerMonth.from_project_dirs(
+                    project_dir_list, WorktimeColumn.MONITORED_WORKTIME_HOUR
+                )
+                project_monitored_worktime.to_csv(root_output_dir / "プロジェクごとの毎月の計測作業時間.csv")
+
             else:
                 logger.warning(f"出力した統計情報は0件なので、`プロジェクトごとの生産性と品質.csv`を出力しません。")
 
@@ -595,8 +536,8 @@ def parse_args(parser: argparse.ArgumentParser):
         help="集計対象のタスクのtask_idを指定します。\n" + "``file://`` を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。",
     )
 
-    parser.add_argument("--start_date", type=str, help="指定した日付（ ``YYYY-MM-DD`` ）以降に教師付を開始したタスクを集計する。")
-    parser.add_argument("--end_date", type=str, help="指定した日付（ ``YYYY-MM-DD`` ）以前に更新されたタスクを集計する。")
+    parser.add_argument("--start_date", type=str, help="指定した日付（ ``YYYY-MM-DD`` ）以降に教師付を開始したタスクから生産性を算出します。")
+    parser.add_argument("--end_date", type=str, help="指定した日付（ ``YYYY-MM-DD`` ）以前に更新されたタスクから生産性を算出します。")
 
     parser.add_argument(
         "--latest",
